from gopay.http import Browser, null_logger
from gopay.api import GoPay, add_defaults
from gopay.oauth2 import OAuth2, InMemoryTokenCache, CachedAuth
from gopay.payments import Payments
from gopay.enums import Language, TokenScope


def payments(config: dict, services: dict = None) -> Payments:
    config = add_defaults(
        config, {"scope": TokenScope.ALL, "language": Language.ENGLISH, "timeout": 30}
    )
    services = add_defaults(
        services, {"logger": null_logger, "cache": InMemoryTokenCache()}
    )
    browser = Browser(services["logger"], config["timeout"])
    gopay = GoPay(config, browser)
    auth = CachedAuth(OAuth2(gopay), services["cache"])
    return Payments(gopay, auth)



from typing import Dict

from gopay.http import Request, Response, Browser
from gopay.enums import Language
import json

JSON = "application/json"
FORM = "application/x-www-form-urlencoded"


class GoPay:
    def __init__(self, config: dict, browser: Browser) -> None:
        self.browser = browser
        self.config = config

    def url(self, path: str):
        if "gatewayUrl" in self.config:
            host = self.config["gatewayUrl"]
            if host.endswith("/"):
                host = host[:-1]
            if not host.endswith("/api"):
                host += "/api"
            host += "/"
            return host + path
        host = (
            "https://gate.gopay.cz/api/"
            if self.config["isProductionMode"]
            else "https://gw.sandbox.gopay.com/api/"
        )
        return host + path

    def call(
        self, url: str, content_type: str, authorization: str, data: Dict
    ) -> Response:
        request = Request()
        request.url = self.url(url)

        request.headers = {
            "Accept": "application/json",
            "Accept-Language": "cs-CZ"
            if self.config["language"] in [Language.CZECH, Language.SLOVAK]
            else "en-US",
            "Authorization": authorization,
        }
        if content_type:
            request.headers["Content-Type"] = content_type

        if data is None:
            request.method = "get"
        else:
            request.method = "post"
            request.body = json.dumps(data) if content_type == JSON else data
        return self.browser.browse(request)


def add_defaults(data: dict, defaults: dict) -> dict:
    full = defaults.copy()
    if data is not None:
        full.update(data)
    return full



class Recurrence:
    DAILY = "DAY"
    WEEKLY = "WEEK"
    MONTHLY = "MONTH"
    ON_DEMAND = "ON_DEMAND"


class Currency:
    CZECH_CROWNS = "CZK"
    EUROS = "EUR"
    POLISH_ZLOTY = "PLN"
    HUNGARIAN_FORINT = "HUF"
    BRITISH_POUND = "GBP"
    US_DOLLAR = "USD"
    ROMANIAN_LEU = "RON"
    BULGARIAN_LEV = "BGN"


class TokenScope:
    CREATE_PAYMENT = "payment-create"
    ALL = "payment-all"


class Result:
    ACCEPTED = "ACCEPTED"
    FINISHED = "FINISHED"
    FAILED = "FAILED"


class PaymentStatus:
    CREATED = "CREATED"
    PAYMENT_METHOD_CHOSEN = "PAYMENT_METHOD_CHOSEN"
    PAID = "PAID"
    AUTHORIZED = "AUTHORIZED"
    CANCELED = "CANCELED"
    TIMEOUTED = "TIMEOUTED"
    REFUNDED = "REFUNDED"
    PARTIALLY_REFUNDED = "PARTIALLY_REFUNDED"


class Language:
    CZECH = "CS"
    ENGLISH = "EN"
    SLOVAK = "SK"
    GERMAN = "DE"
    RUSSIAN = "RU"
    POLISH = "PL"
    HUNGARIAN = "HU"
    FRENCH = "FR"
    ROMANIAN = "RO"
    BULGARIAN = "BG"
    CROATIAN = "HR"
    ITALIAN = "IT"
    SPANISH = "ES"
    UKRAINIAN = "UK"
    ESTONIAN = "EE"
    LITHUANIAN = "LT"
    LATVIAN = "LV"
    SLOVENIAN = "SI"
    PORTUGUESE = "PT"


class PaymentInstrument:
    PAYMENT_CARD = "PAYMENT_CARD"
    BANK_ACCOUNT = "BANK_ACCOUNT"
    PREMIUM_SMS = "PRSMS"
    MPAYMENT = "MPAYMENT"
    PAYSAFECARD = "PAYSAFECARD"
    SUPERCASH = "SUPERCASH"
    GOPAY = "GOPAY"
    PAYPAL = "PAYPAL"
    BITCOIN = "BITCOIN"
    GPAY = "GPAY"
    ACCOUNT = "ACCOUNT"
    APPLE_PAY = "APPLE_PAY"
    CLICK_TO_PAY = "CLICK_TO_PAY"


class StatementGeneratingFormat:
    XLS_A = "XLS_A"
    XLS_B = "XLS_B"
    XLS_C = "XLS_C"
    CSV_A = "CSV_A"
    CSV_B = "CSV_B"
    CSV_C = "CSV_C"
    CSV_D = "CSV_D"
    CSV_E = "CSV_E"
    ABO_A = "ABO_A"


class ItemType:
    ITEM = "ITEM"
    DISCOUNT = "DISCOUNT"
    DELIVERY = "DELIVERY"
    POSTAGE = "POSTAGE"



class EETReceiptState:
    CREATED = "CREATED"
    DELIVERY_FAILED = "DELIVERY_FAILED"
    DELIVERED = "DELIVERED"


class EETMode:
    AUTO = "AUTO"
    EET = "EET"


class BankSwiftCode:
    # CZ
    CESKA_SPORITELNA = "GIBACZPX"
    KOMERCNI_BANKA = "KOMBCZPP"
    CSOB = "CEKOCZPP"
    RAIFFEISENBANK = "RZBCCZPP"
    UNICREDIT_BANK_CZ = "BACXCZPP"
    MONETA_MONEY_BANK = "AGBACZPP"
    FIO_BANKA = "FIOBCZPP"
    MBANK = "BREXCZPP"
    AIRBANK = "AIRACZPP"
    ING_BANK = "INGBCZPP"
    OBERBANK = "OBKLCZ2X"
    VUB_PRAHA = "SUBACZPP"
    HELLO_BANK = "BPPFCZP1"
    CREDITAS = "CTASCZ22"
    MAX_BANKA = "EXPNCZPP"
    JT_BANKA = "JTBPCZPP"
    # SK
    TATRA_BANKA = "TATRSKBX"
    VSEOBECNA_VEROVA_BANKA_BANKA = "SUBASKBX"
    UNICREDIT_BANK_SK = "UNCRSKBX"
    SLOVENSKA_SPORITELNA = "GIBASKBX"
    CSOB_SK = "CEKOSKBX"
    POSTOVA_BANKA = "POBNSKBA"
    OTP_BANKA_SLOVENSKO = "OTPVSKBX"
    PRIMA_BANKA_SLOVENSKO = "KOMASK2X"
    CITIBANK_EUROPE = "CITISKBA"
    FIO_BANKA_SK = "FIOZSKBA"
    MBANK_SK = "BREXSKBX"
    ING_BANK_SK = "INGBSKBX"
    JT_BANKA_SK = "JTBPSKBA"
    OBERBANK_SK = "OBKLSKBA"
    PRIVATBANKA = "BSLOSK22"
    BKS_BANK = "BFKKSKBB"
    RAIFFEISENBANK_SK = "TATRSKBXXXX"
    KOMERCNI_BANKA_SK = "KOMBSKBA"
    # PL
    MBANK1 = "BREXPLPW"
    CITI_HANDLOWY = "CITIPLPX"
    IKO = "BPKOPLPW-IKO"
    INTELIGO = "BPKOPLPW-INTELIGO"
    PLUS_BANK = "IVSEPLPP"
    BANK_BPH_SA = "BPHKPLPK"
    TOYOTA_BANK = "TOBAPLPW"
    VOLKSWAGEN_BANK = "VOWAPLP1"
    SGB = "GBWCPLPP"
    POCZTOWY_BANK = "POCZPLP4"
    BGZ_BANK = "GOPZPLPW"
    IDEA = "IEEAPLPA"
    BPS = "POLUPLPR"
    GETIN_ONLINE = "GBGCPLPK-GIO"
    BLIK = "GBGCPLPK-BLIK"
    NOBLE_BANK = "GBGCPLPK-NOB"
    ORANGE = "BREXPLPW-OMB"
    BZ_WBK = "WBKPPLPP"
    RAIFFEISEN_BANK_POLSKA_SA = "RCBWPLPW"
    POWSZECHNA_KASA_OSZCZEDNOSCI_BANK_POLSKI_SA = "BPKOPLPW"
    ALIOR_BANK = "ALBPPLPW"
    ING_BANK_SLASKI = "INGBPLPW"
    PEKAO_SA = "PKOPPLPW"
    GETIN_ONLINE1 = "GBGCPLPK"
    BANK_MILLENNIUM = "BIGBPLPW"
    BANK_OCHRONY_SRODOWISKA = "EBOSPLPW"
    BNP_PARIBAS_POLSKA = "PPABPLPK"
    CREDIT_AGRICOLE = "AGRIPLPR"
    DEUTSCHE_BANK_POLSKA_SA = "DEUTPLPX"
    DNB_NORD = "DNBANOKK"
    E_SKOK = "NBPLPLPW"
    EUROBANK = "SOGEPLPW"
    POLSKI_BANK_PRZEDSIEBIORCZOSCI_SPOLKA_AKCYJNA = "PBPBPLPW"
    # Others
    SPECIAL = "OTHERS"


class RecurrenceState:
    REQUESTED = "REQUESTED"
    STARTED = "STARTED"
    STOPPED = "STOPPED"


class PreAuthState:
    REQUESTED = "REQUESTED"
    AUTHORIZED = "AUTHORIZED"
    CAPTURED = "CAPTURED"
    CANCELED = "CANCELED"


class PaymentSubStatus:
    _101 = "_101"
    _102 = "_102"
    _3001 = "_3001"
    _3002 = "_3002"
    _3003 = "_3003"
    _5001 = "_5001"
    _5002 = "_5002"
    _5003 = "_5003"
    _5004 = "_5004"
    _5005 = "_5005"
    _5006 = "_5006"
    _5007 = "_5007"
    _5008 = "_5008"
    _5009 = "_5009"
    _5010 = "_5010"
    _5011 = "_5011"
    _5012 = "_5012"
    _5013 = "_5013"
    _5014 = "_5014"
    _5015 = "_5015"
    _5016 = "_5016"
    _5017 = "_5017"
    _5018 = "_5018"
    _5019 = "_5019"
    _5020 = "_5020"
    _5021 = "_5021"
    _5022 = "_5022"
    _5023 = "_5023"
    _5024 = "_5024"
    _5025 = "_5025"
    _5026 = "_5026"
    _5027 = "_5027"
    _5028 = "_5028"
    _5029 = "_5029"
    _5030 = "_5030"
    _5031 = "_5031"
    _5033 = "_5033"
    _5035 = "_5035"
    _5036 = "_5036"
    _5037 = "_5037"
    _5038 = "_5038"
    _5039 = "_5039"
    _5040 = "_5040"
    _5041 = "_5041"
    _5042 = "_5042"
    _5043 = "_5043"
    _5044 = "_5044"
    _5045 = "_5045"
    _5046 = "_5046"
    _5047 = "_5047"
    _5048 = "_5048"
    _5049 = "_5049"
    _5050 = "_5050"
    _6001 = "_6001"
    _6002 = "_6002"
    _6003 = "_6003"
    _6004 = "_6004"
    _6005 = "_6005"
    _6501 = "_6501"
    _6502 = "_6502"
    _6504 = "_6504"


class SupercashSubType:
    SUB_TYPE_PREPAID = "PREPAID"
    SUB_TYPE_POSTPAID = "POSTPAID"



from __future__ import annotations
from typing import Dict, List, Union


from gopay.api import JSON, FORM, add_defaults, Response, GoPay
from gopay.oauth2 import OAuth2


paymentSessionId = Union[List[str], str]


class Payments:
    def __init__(self, gopay: GoPay, oauth: OAuth2) -> None:
        self.gopay = gopay
        self.oauth = oauth

    def create_payment(self, payment: Dict) -> Response:
        payment = add_defaults(
            payment,
            {
                "target": {"type": "ACCOUNT", "goid": self.gopay.config["goid"]},
                "lang": self.gopay.config["language"],
            },
        )
        return self._api("payments/payment", JSON, payment)

    def get_status(self, id_payment: Union[int, str]) -> Response:
        return self._api(f"payments/payment/{id_payment}", FORM, None)

    def refund_payment(self, id_payment: Union[int, str], amount: int) -> Response:
        return self._api(
            f"payments/payment/{id_payment}/refund", FORM, {"amount": amount}
        )

    def refund_payment_eet(
        self, id_payment: Union[int, str], payment_data: Dict
    ) -> Response:
        return self._api(f"payments/payment/{id_payment}/refund", JSON, payment_data)

    def create_recurrence(self, id_payment: Union[int, str], payment: Dict) -> Response:
        return self._api(
            f"payments/payment/{id_payment}/create-recurrence", JSON, payment
        )

    def void_recurrence(self, id_payment: Union[int, str]) -> Response:
        return self._api(f"payments/payment/{id_payment}/void-recurrence", FORM, {})

    def capture_authorization(self, id_payment: Union[int, str]) -> Response:
        return self._api(f"payments/payment/{id_payment}/capture", FORM, {})

    def capture_authorization_partial(
        self, id_payment: Union[int, str], capture_payment: Dict
    ) -> Response:
        return self._api(
            f"payments/payment/{id_payment}/capture", JSON, capture_payment
        )

    def void_authorization(self, id_payment: Union[int, str]) -> Response:
        return self._api(f"payments/payment/{id_payment}/void-authorization", FORM, {})
    
    def get_card_details(self, card_id: int | str) -> Response:
        return self._api(f"payments/cards/{card_id}", FORM, None)

    def get_payment_instruments(
        self, go_id: Union[int, str], currency: str
    ) -> Response:
        return self._api(
            f"eshops/eshop/{go_id}/payment-instruments/{currency}", "", None
        )

    def get_account_statement(self, account_statement: Dict) -> Response:
        return self._api("accounts/account-statement", JSON, account_statement)

    def get_eet_receipt_by_payment_id(self, id_payment: Union[int, str]) -> Response:
        return self._api(f"payments/payment/{id_payment}/eet-receipts", JSON, None)

    def find_eet_receipts_by_filter(self, filter: Dict) -> Response:
        return self._api("eet-receipts", JSON, filter)

    def get_supercash_coupon_batch_status(self, batch_id: Union[int, str]) -> Response:
        return self._api(f"batch/{batch_id}", FORM, None)

    def get_supercash_coupon_batch(self, batch_id: Union[int, str]) -> Response:
        return self._api(
            f'/supercash/coupon/find?batch_request_id={batch_id}&go_id={self.gopay.config["goid"]}',
            FORM,
            None,
        )

    def find_supercash_coupons(self, paymentSessionId: paymentSessionId) -> Response:
        if type(paymentSessionId) is list:
            ids_string = ",".join(map(str, paymentSessionId))
        else:
            ids_string = str(paymentSessionId)
        return self._api(
            f'supercash/coupon/find?payment_session_id_list={ids_string}&go_id={self.gopay.config["goid"]}',
            FORM,
            None,
        )

    def get_supercash_coupon(self, coupon_id: Union[int, str]) -> Response:
        return self._api(f"supercash/coupon/{coupon_id}", FORM, None)

    def url_to_embedjs(self) -> str:
        if "gatewayUrl" in self.gopay.config:
            url_base = self.gopay.config.get("gatewayUrl")
            if not url_base.endswith("/"):
                url_base += "/"
        else:
            url_base = (
                "https://gate.gopay.cz/"
                if self.gopay.config.get("isProductionMode")
                else "https://gw.sandbox.gopay.com/"
            )
        return url_base + "gp-gw/js/embed.js"

    def _api(self, url: str, content_type: str, data: dict) -> Response:
        token = self.oauth.authorize()
        if token.token:
            return self.gopay.call(url, content_type, "Bearer " + token.token, data)
        else:
            return token.response



from typing import Any
import requests


class Request:
    def __init__(self):
        self.method = "get"
        self.url = ""
        self.headers = {}
        self.body = {}


class Response:
    def __init__(self, raw_body: bytes, json: dict, status_code: int):
        self.raw_body = str(raw_body)
        self.json = json
        self.status_code = status_code

    def has_succeed(self) -> bool:
        return self.status_code == 200

    def __str__(self) -> str:
        return self.raw_body


class Browser:
    def __init__(self, logger: Any, timeout: int) -> None:
        self.logger = logger
        self.timeout = timeout

    def browse(self, request: Request):
        try:
            r = requests.request(
                request.method,
                request.url,
                headers=request.headers,
                data=request.body,
                timeout=self.timeout,
            )
            response = Response(r.content, r.json(), r.status_code)
        except ValueError as ve:
            response = Response(r.content, None, r.status_code)
        except Exception as e:
            response = Response(e, {}, 500)
        self.logger(request, response)
        return response


def null_logger(*args):
    pass



from base64 import b64encode
from typing import Any
from gopay.api import FORM, GoPay
from datetime import datetime, timedelta


class AccessToken:
    def __init__(self) -> None:
        self.token = None
        self.response = None
        self.expiration_date = None

    def is_expired(self) -> bool:
        return (
            self.token is None
            or not isinstance(self.expiration_date, datetime)
            or self.expiration_date < datetime.now()
        )


class OAuth2:
    def __init__(self, gopay: GoPay) -> None:
        self.gopay = gopay

    def authorize(self) -> AccessToken:
        credentials = (
            self.gopay.config["clientId"] + ":" + self.gopay.config["clientSecret"]
        )
        token = AccessToken()
        token.response = self.gopay.call(
            "/oauth2/token",
            FORM,
            "Basic " + b64encode(credentials.encode("utf-8")).decode("utf-8"),
            {"grant_type": "client_credentials", "scope": self.gopay.config["scope"]},
        )
        if token.response.has_succeed():
            token.token = token.response.json["access_token"]
            token.expiration_date = datetime.now() + timedelta(
                seconds=token.response.json["expires_in"]
            )
        return token

    def get_client(self) -> str:
        return "-".join(
            [
                self.gopay.config["clientId"],
                self.gopay.url(""),
                self.gopay.config["scope"],
            ]
        )


class InMemoryTokenCache:
    def __init__(self):
        self.tokens = {}

    def get_token(self, client: str) -> AccessToken:
        return self.tokens.get(client)

    def set_token(self, client: str, token: AccessToken) -> None:
        self.tokens[client] = token


class CachedAuth:
    def __init__(self, oauth: OAuth2, cache: Any) -> None:
        self.oauth = oauth
        self.cache = cache

    def authorize(self) -> AccessToken:
        client = self.oauth.get_client()
        token = self.cache.get_token(client)
        if not isinstance(token, AccessToken) or token.is_expired():
            token = self.oauth.authorize()
            self.cache.set_token(client, token)
        return token
